package ca.ubc.cs.elide.nodes;

import ca.ubc.cs.elide.*;
import ca.ubc.cs.elide.visitors.*;
import ca.ubc.cs.elide.utils.*;

import java.util.*;
import java.lang.reflect.*;

/**
 * A <code>MethodNode</code> provides information about a single method on a class or interface.
 * The method may be a class method or an instance method. <p>
 * The easiest way to create a new <code>MethodNode</code> is with {@link ClassNode#extend} or
  * {@link NodeFactory#createMethod}.
 **/
public class MethodNode extends CodeBearingNode
{		
	private String returnType = "";
		
	public MethodNode() {}
	public MethodNode(String name) { super(name); }
	
	/**
	 * Returns the the name of the formal return type of the method represented by this
	 * <code>MethodNode</code> object.
	 **/
	public String getReturnType() { return returnType; }
	
	/**
	 * Sets the name of the formal return type of the method represented by this <code>MethodNode</code> object.
	 **/
	public void setReturnType(String returnType) { this.returnType = returnType; }

	/**
	 * Returns an array of <code>ParameterNode</code> objects, representing the formal parameters of the method represented
	 * by this <code>MethodNode</code> object.
	 **/
	public ParameterNode[] getParameters() { return (ParameterNode[]) getChildren(ParameterNode.class); }
	
	/** Returns an array of <code>TypeNode</code> objects, representing the types of exception declared to be thrown from the method
	 * represented by this object.
	 */
	public TypeNode[] getExceptionTypes() {	return (TypeNode[]) getChildren(TypeNode.class); }
		
	/**
	 * Returns a <code>ParameterNode</code> object for a formal parameter of the method represented by this object, given its name.  Returns null if no such
	 * formal parameter exists.
	 **/
	public ParameterNode getParameter(String name) { return (ParameterNode) getChild(ParameterNode.class, name); }
	
	/**
	 * Returns <code>true</code> if the method represented by this <code>MethodNode</code> object is <code>abstract</code>,
	 * <code>native</code>, or is declared in an interface rather than a <code>class</code>.  Returns false otherwise.
	 **/
	public boolean isBodyless()
	{
			return ((Modifier.isAbstract(getModifiers())) ||
					(Modifier.isNative(getModifiers())) ||
					getDeclaringClass().isInterface());
	}
		
	boolean isAddableTo(ChildBearingNode node)
	{
		return (node instanceof ClassNode);
	}
}