package ca.ubc.cs.elide.nodes;

import ca.ubc.cs.elide.*;
import ca.ubc.cs.elide.visitors.*;
import ca.ubc.cs.elide.utils.*;

import java.util.*;
import java.lang.reflect.Modifier;

/** <code>ModifierBearingNode</code> is an abstract class, representing a Java element that can have modifiers such
 * as <code>public</code> and <code>synchronized</code> attached to it.
 **/
 
public abstract class ModifierBearingNode extends ChildBearingNode
{	
	public ModifierBearingNode() {}
	public ModifierBearingNode(String name) { super(name); }

	private int modifiers = 0;
	
	/**
	 * Returns the modifiers attached to this node, as an integer.  Use {@link java.lang.reflect.Modifier} to decode.
	 **/
	 
	public int getModifiers() { return modifiers; }
	
	/**
	 * Sets the access on this node to the default (package) access.  Removes any other access modifiers.
	 **/
	public void makeDefaultAccess()
	{
		modifiers &= ~ Modifier.PUBLIC;
		modifiers &= ~ Modifier.PRIVATE;
		modifiers &= ~ Modifier.PROTECTED;
	}
	
	/**
	 * Sets the access on this node to public access.  Removes any other access modifiers.
	 **/
	public void makePublic()
	{
		makeDefaultAccess();
		modifiers |= Modifier.PUBLIC;
	}
	
	/**
	 * Sets the access on this node to private access.  Removes any other access modifiers.
	 **/
	public void makePrivate()
	{
		makeDefaultAccess();
		modifiers |= Modifier.PRIVATE;
	}

	/**
	 * Sets the access on this node to protected access.  Removes any other access modifiers.
	 **/
	public void makeProtected()
	{
		makeDefaultAccess();
		modifiers |= Modifier.PROTECTED;
	}

	/**
	 * Sets the <code>static</code> property of this node; <code>true</code> to add <code>static</code>,
	 * <code>false</code> to remove it.
	 **/
	public void setStatic(boolean b) { setModifier(Modifier.STATIC, b); }

	/**
	 * Sets the <code>synchronized</code> property of this node; <code>true</code> to add <code>synchronized</code>,
	 * <code>false</code> to remove it.
	 **/
	 public void setSynchronized(boolean b) { setModifier(Modifier.SYNCHRONIZED, b); }
	/**
	 * Sets the <code>native</code> property of this node; <code>true</code> to add <code>native</code>,
	 * <code>false</code> to remove it.
	 **/
	 public void setNative(boolean b) { setModifier(Modifier.NATIVE, b); }
	/**
	 * Sets the <code>static</code> property of this node; <code>true</code> to add <code>static</code>,
	 * <code>false</code> to remove it.
	 **/
	 public void setFinal(boolean b) { setModifier(Modifier.FINAL, b); }
	/**
	 * Sets the <code>static</code> property of this node; <code>true</code> to add <code>static</code>,
	 * <code>false</code> to remove it.
	 **/
	 public void setTransient(boolean b) { setModifier(Modifier.TRANSIENT, b); }
	/**
	 * Sets the <code>static</code> property of this node; <code>true</code> to add <code>static</code>,
	 * <code>false</code> to remove it.
	 **/
	 public void setVolatile(boolean b) { setModifier(Modifier.VOLATILE, b); }
	/**
	 * Sets the <code>static</code> property of this node; <code>true</code> to add <code>static</code>,
	 * <code>false</code> to remove it.
	 **/
	 public void setAbstract(boolean b) { setModifier(Modifier.ABSTRACT, b); }
	
	private void setModifier(int mod, boolean on)
	{
		if(on)
			modifiers |= mod;
		else
			modifiers &= ~mod;
	}
	
	boolean isAddableTo(ChildBearingNode node)
	{
		return true;
	}
}