package ca.ubc.cs.elide.nodes;
import ca.ubc.cs.elide.sable.*;
import ca.ubc.cs.elide.utils.*;

/**
 * Use <code>NodeFactory</code> to easily create instances of
 * {@link ClassNode}, {@link FieldNode}, {@link MethodNode}, and {@link TextNode}.
 *<p>
 * For each of these classes, <code>NodeFactory</code> takes a string with source
 * code describing the object, and returns the parsed representation.
 * Often, it will be easiest to use ELIDE's extended
 * %{}% string literal syntax with these methods.
 **/
public class NodeFactory
{
	/**
	 * Creates a <code>ClassNode</code> from a String of Java source code
	 * defining the class.
	* <p> For example: <p>
	* <pre>
	* ClassNode newClass = NodeFactory.createClass(%{
	* 	public class Foo {
	*		...
	*	}
	*   }%);
	* </pre>
	**/
	
	public static ClassNode createClass(String text)
	{
		StartNode start = SableAdaptor.compileText(text);
		return start.getDefaultPackage().getClasses()[0];		
	}
	
	/**
	 * Creates a <code>FieldNode</code> from a String of Java source code
	 * defining the field.
	* <p> For example: <p>
	* <pre>
	* FieldNode newField = NodeFactory.createField(%{ public int x = 3 }%);
	* </pre>
	* <p>
	* In most cases, {@link ClassNode#extend} can be used instead.
	**/
	public static FieldNode createField(String text)
	{
		ClassNode klass = createClass("class extension { " + text + ";}");
		return klass.getDeclaredFields()[0];
	}
	
	/**
	 * Creates a <code>MethodNode</code> from a String of Java source code
	 * defining the class.
	* <p> For example: <p>
	* <pre>
	* MethodNode newMethod = NodeFactory.createMethod(%{
	* 	public void foo() {
	*		...
	*	}
	*   }%);
	* </pre>
	*
	* In most cases, {@link ClassNode#extend} can be used instead.
	**/
	public static MethodNode createMethod(String text)
	{
		ClassNode klass = createClass("class extension { " + text + "}");
		return klass.getDeclaredMethods()[0];
	}
	
	/**
	 * Creates a <code>TextNode</code> from a String of Java source code.
	* <p> For example: <p>
	* <pre>
	* TextNode snippet = NodeFactory.createText(%{
	* 	if(x > 0) { doSomething(); }
	*   }%);
	* </pre>
	**/
	public static TextNode createText(String text)
	{
		MethodNode method = createMethod("extension() {"+text+"}");
		return (TextNode) method.getCodeElements()[0];
	}	
}